#  Copyright (c) 2022. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.
import io
from abc import ABCMeta, abstractmethod
from enum import Enum


class Writer(metaclass=ABCMeta):
    @abstractmethod
    def write(self, line: str):
        pass

    @abstractmethod
    def writelines(self, lines: [str]):
        pass


class Reader(metaclass=ABCMeta):
    @abstractmethod
    def readline(self):
        pass


class IO():
    class FileIO:
        def __init__(self, fp: io.TextIOBase):
            self._fp = fp

        def __del__(self):
            self._fp.close()

    class FileWriter(FileIO, Writer):
        def write(self, data):
            """
            ファイルに1行書き込む
            :param data:
            :return: None
            """
            self._fp.write(data)

        def writelines(self, lines: [str]):
            """
            ファイルに複数行書き込む
            :param lines: [str]
            :return: None
            """
            self._fp.writelines(lines)

    class FileReader(FileIO, Reader):
        def readline(self) -> str:
            return self._fp.readline()

    class Encoding(Enum):
        """ ファイルエンコーディング """
        CP932 = 'cp932'
        """ Windows Shift-JIS """
        UTF8_WITH_NO_BOM = 'utf8'
        UTF8_WITH_BOM = 'utf8b'

    class NewLine(Enum):
        '''
        Windowsの改行コード
        '''
        CRLF = "\r\n"
        LF = "\n"
        CR = "\r"

    @classmethod
    def open_for_read(cls, name: str, *, encoding: Encoding, newline: NewLine) -> Reader:
        fp = open(name, mode="r", encoding=encoding.value, newline=newline.value)
        obj = IO.FileWriter(fp)
        return obj

    @classmethod
    def open_for_write(cls, name: str, *, encoding: Encoding, newline: NewLine) -> Writer:
        fp = open(name, mode="w", encoding=encoding.value, newline=newline.value)
        obj = IO.FileWriter(fp)
        return obj
